/*
    This file is part of MORPHEUS.

    MORPHEUS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    MORPHEUS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "sound.h"

#define FREQ(X) (F_CPU/(2*(X)-1))

#define BASE 440

#define C0      1.0
#define D0      1.122
#define E0      1.26
#define FIS0    1.414
#define G0      1.498
#define A0      1.682
#define H0      1.888
#define C1      2*1.0
#define D1      2*1.122

#define SOUND_ON  do { TCCR1A = (1 << COM1B1) | (1 << COM1A1) | (1 << COM1A0); } while(0)
#define SOUND_OFF do { TCCR1A = 0; } while(0)

typedef struct
{
    uint16_t freq;
    uint8_t delay;
    int8_t anz;
    int16_t inc;
} ton_t;

typedef struct sound_t
{
    uint8_t delay;
    int8_t anz;
    ton_t ton;
} sound_t;

const ton_t TON[] PROGMEM = 
{
    [SOUND_SNAKE_EAT] = { FREQ(700),  1, 25,  -300},
    [SOUND_SNAKE_DIE] = { FREQ(650),  2, 25,  1200},
    [SOUND_ASTRO_SHIP_SHOOT] = { FREQ(1000), 2, 15,  1800},
    [SOUND_ASTRO_UFO_SHOOT]  = { FREQ(2400), 1, 15,  2400},
    [SOUND_ASTRO_SHIP_CRASH] = { FREQ(2400), 1, 60,  17343}
};

sound_t sdata;

void 
sound (uint8_t id)
{
    memcpy_P (&sdata.ton, &TON[id], sizeof (ton_t));
    sdata.delay = sdata.ton.delay;
    sdata.anz   = sdata.ton.anz;
}

void 
job_sound (sound_t * s)
{
    uint8_t delay = s->delay;
    
    if (0 == delay)
        // Nichts zu tun
        return;
        
    if (--delay)
    {
        // delay noch nicht abgelaufen
        s->delay = delay;
        return;
    }
    
    // delay abgelaufen
    
    int8_t anz = s->anz;
    
    if (0 == anz)
        return;
        
    if (!--anz)
    {
        // anz abgelaufen: sound ist beendet
        s->delay = 0;
        s->anz   = 0;
        SOUND_OFF;
        return;
    }

    // delay aufziehen
    s->delay = s->ton.delay;
    s->anz = anz;

    // PWM-Frequenz fr Ton setzen, Duty = 50%
    uint16_t icr = s->ton.freq;
    s->ton.freq = icr + s->ton.inc;
    
    ICR1 = icr;   
    OCR1A = icr/2;
    OCR1B = icr/2;
    
    SOUND_ON;
}

void timer1_init(void)
{
    DDRB |= (_BV(5)|_BV(6));	//MAKE_OUT (PORT_SPKA); MAKE_OUT (PORT_SPKB);
    PORTB |= (_BV(5)|_BV(6));	//SET (PORT_SPKA); SET (PORT_SPKB);

    // Mode #8 fr Timer1 (Manual S. 97/ATmega8, S.133/ATmega168)
    // PWM, Phase & Freq correct, TOP = ICR1
    // und volle MCU-Clock
    TCCR1A = 0;
    TCCR1B = (1 << WGM13) | (1 << CS10);

#if 1
    // ICR1 fr gewnschte Frequenz
    uint16_t icr1 = FREQ (930);

    ICR1 = icr1;
    // Tastverhltnis = 50%
    OCR1A = icr1/2;
    OCR1B = icr1/2;
#endif
}
