; ChangeMenu.asm -- Included source for Dutchtronix Oscilloscope Clock
;
;  Copyright  2008 Jan P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Firmware for Dutchtronix AVR Oscilloscope Clock 
;
; Check Time/Date update using push button state
;
; Update modes: Hands, Binary, Numeric Time, Numeric Date
;
; Time Update mode:
; 	Stops the clock! Uses "Push to start" once done
;	Called from special loop in MainLoop to prevent user commands from serial port.
;
; Date Update Mode:
;	Time continues but numeric date field is NOT updated while CMod state active
;

CModStateAdvance:
	ldiw	Z,CModStateTable*2		;state actions table
	ldd		r16,CModState			;get current state
	rcall	StateCheckNOPUSH		;may set fUPS1
	std		CModState,r16			;save result
#if DEBUG
	clr		r17
#endif
CModSA50:
	lpm		r0,Z+
	cp		r0,r16					;current state matches table entry?
	breq	CModSA100				;yes
	adiw	ZL,5					;next table entry
#if DEBUG
	inc		r17
	cpi		r17,((CModStateTableEnd-CModStateTable)*2/6)+1
	brlo	CModSA50
	rjmp	PC
#endif
	rjmp	CModSA50
CModSA100:
;
; we've located the table entry for the current state
; see if a short push was issued
;
	sbrs	G_FLAGS,fUPS1
	rjmp	CModSA150			;no
;
; short push: advance field value
;
	cbr		G_FLAGS,(1<<fUPS1)	;clear
	adiw	Z,1					;advance to Short Push action ptr
	lpm		r16,Z+				;get Short Push action ptr
	lpm		r17,Z
	movw	ZL,r16				;Z no longer needed
	icall						;increment value/Short Push Action
	rjmp	CModSA900			;done
;
; see if a long push was issued (could be an artificial one)
;
CModSA150:
#if USESW2
	sbis	GPIOR0,fUPS2		;S2 fUp flag (S2 short push)
	rjmp	CModSA1000			;clear - continue loop
	cbi		GPIOR0,fUPS2		;clear fUp flag
CModSA200:
#else
	sbrs	G_FLAGS,fTC0S1		;TC0 flag (long push)
	rjmp	CModSA1000			;no, done
#endif
;
; advance to the next field
;
	lpm		r16,Z+				;get next state
	rcall	StateCheckNOPUSH	;may set fUPS1
	rcall	NextCModState
	CALL	StartTC1Countdown
	adiw	Z,2					;skip Short Push Action
	lpm		r16,Z+				;get field to flash info
	lpm		r17,Z
	mov		r0,r16				;test for NULL
	or		r0,r17
	breq	CModSA900			;nothing to do
	sbrs	r17,STRINGFLASHBIT	;STRINGFLASH Field?
	rjmp	CModSA300			;no, pointer flash
	rcall	SetNumFlashField	;field to flash field in r16
	rjmp	CModSA900			;done
CModSA300:
	sbrs	r17,DOUBLEPTRBIT	;DOUBLEPTR Field?
	rjmp	CModSA400			;no, simple pointer flash
	cbr		r17,(1<<DOUBLEPTRBIT) ;clear flag
	push	r16					;save for StartStateFlash1
	push	r17
	movw	XL,r16				;find second field
	adiw	XL,4				;Next Ptr Field
	rcall	StartStateFlash2
	pop		r17					;restore
	pop		r16
CModSA400:
	movw	XL,r16
	rcall	StartStateFlash1
;
; Start timeout counter
;
CModSA900:
	CALL	StartTC1Countdown
CModSA1000:
	ret
;
; check if the state is a NOPUSH state
;
; IN: r16	state to check
;
; OUT:	r16 updated state, Possibly G_FLAGS modified
;		r17 used
; 
StateCheckNOPUSH:
	mov		r17,r16
	andi	r17,NOPUSH
	breq	StateCheckNO100		;no, continue
	sbr		G_FLAGS,(1<<fUPS1)	;fake a short push
	andi	r16,~NOPUSH
StateCheckNO100:
	ret
;
; display "PUSH TO START"
;
SetPushToStart:
	ldiw	X,ScanTbl_Str1		;String Display Field
	ldi		r16,low(TPushToStart*2)
	st		X+,r16
	ldi		r16,high(TPushToStart*2)
	st		X+,r16
	ldi		r16,P2SHPOS
	st		X+,r16
	ldi		r16,P2SVPOS
	st		X+,r16
	sbiw	X,4					;Point to field to Flash
	ret
;
; Note. StartStateTextFlash setups the numFlashDispBuf in the
;		flash tables, even though numFlashDispBuf has not been
;		initialized yet. Setting fTC0S1/fUPS2 causes SetNumFlashField
;		to be called, which does the initialization before
;		the field is being flashed (hopefully)
;
StateSetHRNR:
StateSetYEAR:
	rcall	StartStateTextFlash	;Numeric Text String to flash
StateSetHRHAND:
StateSetHRBIN:
	rjmp	StateSetCommon

StateSetMODSET:
	rcall	SetPushToStart
	rcall	StopStateTextFlash
StateSetCommon:
#if USESW2
;
; now fake a short S2 push to advance to the next state
;
	sbi		GPIOR0,fUPS2		;S2 fUp flag (S2 short push)
#else
;
; now fake a long S1 push to advance to the next state
;
	sbr		G_FLAGS,(1<<fTC0S1)
#endif
	ret
;
; Increment Seconds
;
StateIncSec:
	ldd		r16,AvrSecs
	inc		r16
	cpi		r16,60				;in range?
	brlo	StateIncSecL100
	clr		r16
StateIncSecL100:
	std		AvrSecs,r16
	rjmp	StateIncCommon
;
; Increment Minutes
;
StateIncMin:
	ldd		r16,AvrMins
	inc		r16
	cpi		r16,60				;in range?
	brlo	StateIncMinL100
	clr		r16
StateIncMinL100:
	std		AvrMins,r16
	rjmp	StateIncCommon
;
; Increment Hours
;
StateIncHr:
	ldd		r16,AvrHrs
	inc		r16
	cpi		r16,24
	brlo	StateIncHrL100
	clr		r16					;wrap
StateIncHrL100:
	std		AvrHrs,r16
StateIncCommon:
	rcall	UpdateDisplay
	ret

StateSetTime:
	RCALL	RTCSetTime			;update the RTC
;
; check if the new time is in a DST window. If so, set the proper bitvector flag
; but do not apply a DST correction.
;
	CALL	DSTRetroNotApply
	rjmp	StateJoint

CTimeCancel:
	RCALL	RTCReadTime			;reload AVR time from RTC
;	rjmp	StateJoint			;FALL THROUGH

Statejoint:
	ldi		r16,CMODIDLE
	rcall	NextCModState
	clrsw	ScanTbl_Str1		;clear PUSH TO START message
	rcall	UpdateDisplay
	ret

StateIncDay:
	rcall	AVRIncDay
	rjmp	StateIncCommon
;
; Validate the day field based on the newly selected month/year
;
StateSetDay:
	ldd		r16,AvrDay
	rcall	CheckAVRDaysInMonth
	tst		r16					;result
	brne	StateSDL100			;ok
	dec		r17
	std		AvrDay,r17
StateSDL100:
#if USESW2
;
; now fake a short S2 push to advance to the next state
;
	sbi		GPIOR0,fUPS2		;S2 fUp flag (S2 short push)
#else
;
; now fake a long S1 push to advance to the next state
;
	sbr		G_FLAGS,(1<<fTC0S1)
#endif
	ret

StateIncMonth:
	rcall	AVRIncMonth
	rjmp	StateIncCommon

StateIncYear:
	rcall	AVRIncYear
	rjmp	StateIncCommon
;
; Internal Date has been updated by the user.
; Now propagate this to the RTC
;
StateSetDate:
	rcall	StopStateTextFlash
	RCALL	RTCSetDate				;This will validate the new date
;
; check if the new date is in a DST window
;
	CALL	DSTRetroApply		;will update RTC if doing DST fix. Should test
#if USESW2
;
; now fake a short S2 push to advance to the next state
;
	sbi		GPIOR0,fUPS2		;S2 fUp flag (S2 short push)
#else
;
; now fake a long S1 push to advance to the next state
;
	sbr		G_FLAGS,(1<<fTC0S1)
#endif
	ret

CDateCancel:					;if TIMEOUT
	RCALL	RTCReadDate			;reload AVR date from RTC
	rcall	UpdateDisplay
	ret
;
; Table based Cmenu Processing
;
CMenuStateAdvance:
	ldiw	Z,CMenuStateTbl*2
	ldd		r16,CMenuState
#if DEBUG
	cpi		r16,CMENUIDLE
	brne	CMenuSA50
	rjmp	PC
CMenuSA50:
#endif
	cpi		r16,CMENUNAMEDIT	;special UsrNameEdit mode
	brne	CMenuSA80			;no, process actions normally
;
; Special handling of CMENUNAMEDIT mode
;
	lds		r17,MenuUsrNameVal	;currently editing User Name?
	cpi		r17,USRNAMEEDIT
	brne	CMenuSA60			;no, skip this state and exit menu
	rjmp	CMenuSA80			;process actions
CMenuSA60:
;
; This only works if CMENUNAMEDIT is the last Menu state
;
	ldi		r16,CMENUIDLE
	rcall	NextCMenuState
	ret
CMenuSA80:
	subi	r16,CMENUIDLE+1		;rebase
	mov		r0,r16
	lsl		r0					;*4
	lsl		r0
	lsl		r16					;*2
	add		r16,r0				;total *6
	add		ZL,r16
	adc		ZH,_0
;
; preserve Z!
;
	sbrs	G_FLAGS,fUPS1		;short push
	rjmp	CMenuSA100
	cbr		G_FLAGS,(1<<fUPS1)	;clear
	lpm		r0,Z+				;get Action func ptr from Z
	lpm		r1,Z+
	movw	ZL,r0
	icall						;action
	rjmp	CMenuSA200
CMenuSA100:
#if USESW2
	sbis	GPIOR0,fUPS2		;S2 fUp flag (S2 short push)
	rjmp	CMenuSA900			;clear - continue loop
	cbi		GPIOR0,fUPS2		;clear fUp flag
#else
	sbrs	G_FLAGS,fTC0S1		;TC0 flag (long push)
	rjmp	CMenuSA900
#endif
;
; advance to next field
;
CMenuSA110:
	adiw	Z,2					;get Finalize func ptr
	lpm		r0,Z+
	lpm		r1,Z+
	pushw	Z					;save CMenuStateTbl Ptr Point A
	movw	ZL,r0
	icall						;action
CMenuSA150:
	ldd		r16,CMenuState		;next CMenuState
	inc		r16
	cpi		r16,CMENULAST+1
	brlt	CMenuSA175
	ldi		r16,CMENUIDLE
CMenuSA175:
	rcall	NextCMenuState
	popw	Z					;restore CMenuStateTbl ptr from Point A
	lpm		XL,Z+				;get Flash field ptr from Z
	lpm		XH,Z+
	mov		r0,XL				;check for NULL
	or		r0,XH
	breq	CMenuSA200
	rcall	StartMenuStateFlash
CMenuSA200:
	CALL	StartTC1Countdown	;start timeout timer
CMenuSA900:
	ret

CMenuDummy:
CMenuDemo:
#if !DEMOMODE
UsrDemo:
#endif
	ret
;
; Start Text field flashing to show it is being updated
;
; Argument:
;
StartStateTextFlash:
	ldiw	X,ScanTbl_Str4
#if DEBUG
	ld		ZL,X+				;current vector value. Assumed to be NumDispBuf 
	ld		ZH,X+
	sbiw	X,2					;back
	ldi		r16,low(NumDispBuf)
	mov		r0,r16
	ldi		r16,high(NumDispBuf)
	mov		r1,r16
	cp		r0,ZL
	cpc		r1,ZH
	breq	StartSTFL100
	rjmp	PC
StartSTFL100:
#else
	ldiw	Z,NumDispBuf
#endif
	ldi		r16,low(NumFlashDispBuf)
	mov		r0,r16				;Alt Vector Lo
	ldi		r16,high(NumFlashDispBuf)
	mov		r1,r16				;Alt Vector Hi
	ser		r16					;indefinite
	CALL	AddFlashItem
	stdw	CTextFlashPtr,X		;save flash ptr
	ret
;
; Start Menu SRam Text field flashing to show it is being updated
;
; Argument:
;
StartMenuTextFlash:
	ldiw	X,MScanTbl_UsrNameTxt
#if DEBUG
	ld		ZL,X+				;current vector value. Assumed to be UsrNameBuf
	ld		ZH,X+
	sbiw	X,2					;back
	ldi		r16,low(UsrNameBuf)
	mov		r0,r16
	ldi		r16,high(UsrNameBuf)
	mov		r1,r16
	cp		r0,ZL
	cpc		r1,ZH
	breq	StartMTFL100
	rjmp	PC
StartMTFL100:
#else
	ldiw	Z,UsrNameBuf
#endif
	ldi		r16,low(UsrNameFlashBuf)
	mov		r0,r16				;Alt Vector Lo
	ldi		r16,high(UsrNameFlashBuf)
	mov		r1,r16				;Alt Vector Hi
	ser		r16					;indefinite
	CALL	AddFlashItem
	stdw	CTextFlashPtr,X		;save flash ptr
	ret

StopStateTextFlash:
StopMenuTextFlash:
	lddw	X,CTextFlashPtr		;retrieve flash ptr
	CALL	RemoveFlashItem
	clrdw	CTextFlashPtr		;clear ptr
	std		TextFlashField,_0	;clear Buf Field Mod Flag	
	ret

;
; Start field flashing to show it is being updated
;
; IN: X ScanTbl ptr
;
StartStateFlash:
	ld		ZL,X+				;current vector value
	ld		ZH,X+
	sbiw	X,2					;back
	ser		r16					;indefinite
	CALL	AddFlashItemNoAlt
	ret
;
; Start field flashing and preserve Flash Ptr
;
; IN: X ScanTbl ptr
;
StartStateFlash1:
	rcall	StartStateFlash
	stdw	CModFlashPtr1,X		;save flash ptr
	ret

StopStateFlash1:
	lddw	X,CModFlashPtr1		;retrieve flash ptr
	CALL	RemoveFlashItem
	clrdw	CModFlashPtr1		;clear ptr
	ret
;
; Start field flashing and preserve Second Flash Ptr
;
; IN: X ScanTbl ptr
;
StartStateFlash2:
	rcall	StartStateFlash
	stdw	CModFlashPtr2,X		;save flash ptr
	ret

StopStateFlash2:
	lddw	X,CModFlashPtr2		;retrieve flash ptr
	CALL	RemoveFlashItem
	clrdw	CModFlashPtr2		;clear ptr
	ret
;
; r16 holds next state
;
NextCModState:
	cbr		G_FLAGS,(1<<fTC0S1)	;clear
	std		CModState,r16		;move to next state
	rcall	StopStateFlash1
	rcall	StopStateFlash2		;must preserve Z
; Start timeout
;	rcall	StartTC1Countdown
	ret
;
; The vector ptr in the current State Flash Entries
; will be changed as values get updated. Update the vector ptrs
; currently in the Flash Table.
; Do nothing if currently not flashing.
;
; Argument: Z ScanTbl ptr to update
;			r16,r17 new VectorPtr
;
; Uses: X, r20
;
UpdateCModFlashEntry:
	lddw	X,CModFlashPtr1	 ;get First TimeFlashPtr
	CALL	UpdateFlashEntry
	lddw	X,CModFlashPtr2	 ;get Second MenuFlashPtr
	RCALL	UpdateFlashEntry
	ret
;
; Start Menu field flashing to show it is being updated
;
; Argument: X MScanTbl ptr
;
StartMenuStateFlash:
	rcall	StartStateFlash
	stdw	CMenuFlashPtr,X	;save flash ptr
	ret

StopMenuStateFlash:
	lddw	X,CMenuFlashPtr	;retrieve flash ptr
	CALL	RemoveFlashItem
	clrdw	CMenuFlashPtr	;clear ptr
;	std		CMenuFlashPtr+1,_0
	ret
;
; r16 holds next state
;
NextCMenuState:
	cbr		G_FLAGS,(1<<fTC0S1)	;clear
	std		CMenuState,r16		;move to next state
	rcall	StopMenuStateFlash
	ret
;
; The vector ptr in the current State Flash Entries
; will be updated as values get changed. Update the vector ptrs
; currently in the Flash Table.
; Do nothing if currently not flashing.
;
; Argument: Z MScanTbl ptr to update
;			r16,r17 new VectorPtr
;
; Uses: X, r20
;
UpdateCMenuFlashEntry:
	lddw	X,CMenuFlashPtr		;get CMenuFlashPtr
	CALL	UpdateFlashEntry
	ret
;
; Timeout UsrName edit mode
;
TimeOutEditUsrName:
	lds		r17,MenuUsrNameVal		;currently editing User Name?
	cpi		r17,USRNAMEEDIT
	brne	TimeOutEUNL200			;no, nothing to do
#if DEBUG
	ldd		r16,CMenuState			;Verify current state is CMENUNAMEDIT
	cpi		r16,CMENUNAMEDIT
	breq	TimeOutEUNL100
	rjmp	PC						;inconsistent
TimeOutEUNL100:
#endif
	rcall	StopMenuTextFlash		;Restore MScanTbl ptr
	rcall	RestoreUsrName			;Restore original UsrName
TimeOutEUNL200:
	ret

RestoreUsrName:
	rcall	ReadEEUsrName			;Restore original UsrName
	rcall	SetDialDisplay			;re-display user name
	ret

SaveUsrName:
	rcall	WriteEEUsrName			;Also writes UsrNameCnt
	rcall	SetDialDisplay			;re-display user name
	ret
;
; Populate Menu Table Value fields with current values
; Flags retain their original interpretation as much as possible.
;
CMenuSetTable:
	ldd		r16,ShowCalibration
	std		MenuCalVal,r16
	rcall	CMenuSetCalVal

	lds		r17,CurrNumVal
	std		MenuNumVal,r17
	rcall	CMenuSetNumVal

	lds		r16,LedDisabled
	std		MenuLedVal,r16
	rcall	CMenuSetLedVal
	
	lds		r16,BurnInDisabled
	std		MenuBurninVal,r16
	rcall	CMenuSetBurninVal
		
	lds		r16,CurrDialVal
	std		MenuDialVal,r16
	rcall	CMenuSetDialVal

#if FUNMODES
	ldd		r0,CurrPlayMode
	std		MenuPlayVal,r0
	rcall	CMenuSetPlayVal
#endif

#if MULTIAPP
	lds		r0,CurrAppMode
	sts		MenuAppVal,r0
	rcall	CMenuSetAppVal
#endif

	lds		r0,UsrNameFlg
	sts		MenuUsrNameVal,r0
	rcall	CMenuSetUsrNameVal

	lds		r0,ChronoEnabled		;Boolean
	sts		MenuChronoVal,r0
	rcall	CMenuSetChronoVal

	lds		r0,UsrNameCnt
	std		MenuUsrNameCnt,r0
	std		UsrNameEditIdx,_0		;start edit at char 0
	rcall	CMenuCenterUsrName
	rcall	CmenuFlashUsrName

	ldi		r16,NOGPS				;preset
	lds		r0,GPSInEnabled			;GPS currently enabled?
	tst		r0
	breq	CMenuSTL500				;no
	lds		r16,GPSOffset			;get local time zone offset
	subi	r16,-(NOGPS+13)			;rebase to NOGPS range
#if DEBUG
	cpi		r16,GPSOFSPLUS12+1		;Max new value
	brlt	CMenuSTL500
	rjmp	PC
#endif
CMenuSTL500:
#if GPS
	sts		MenuGPSVal,r16
	rcall	CMenuSetGPSVal
#endif

	lds		r16,DSTMode				;Composite Value: US, EU, none
	sts		MenuDSTMode,r16
	rcall	CMenuSetDSTVal

	sts		MenuEEInitVal,_0
	rcall	CMenuSetInitVal

#if MULTIBAUD
	lds		r16,BaudVal				;range 0.. 7
	subi	r16,-BAUDVALBASE
	sts		MenuBaudVal,r16			;range BAUDVAL19200 to BAUDVAL14400
	rcall	CMenuSetBaudVal
	ret
#endif

	ret

;
; Start of Menu Action Functions
;

;
; Next Dial Value Field
; Options: 12HR, 24HR, ROMAN, BINARY, OFF
;
CMenuNextDialVal:
	ldd		r16,MenuDialVal
	inc		r16				;next value
	cpi		r16,DIALMIN+1	;wrap?
	brlt	NextDVL100
	ldi		r16,DIAL12HR	;yes
NextDVL100:
	std		MenuDialVal,r16
;	FALL THROUGH
;
; set MScanTbl Dial Value field based on MenuDialVal
;
CMenuSetDialVal:
	ldd		r16,MenuDialVal
	subi	r16,DIAL12HR		;scale to 0 based
	ldiw	Z,DialValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_DialVal
	rjmp	CommonSetVal

CMenuSaveDialVal:
	ldd		r16,MenuDialVal
	lds		r17,CurrDialVal
	cp		r16,r17
	breq	CMenuSDVL300		;no change
#if !DIAL24
	cpi		r16,DIAL24HR
	brne	CMenuSDV100
	dec		r16					;make it DIAL12HR
CMenuSDV100:
#endif
	rcall	UpdateDialDisplay
	rcall	SetCMENULAST
CMenuSDVL300:
	ret
;
; Next Numeric Value Field
; Options: 12HR, 24HR, DATE, ALARM, OFF
;
CMenuNextNumVal:
	ldd		r16,MenuNumVal
	inc		r16				;next value
	cpi		r16,NUMOFF+1	;wrap?
	brlt	NextNVL100
	ldi		r16,NUM12HR		;yes
NextNVL100:
	std		MenuNumVal,r16
;
; FALL THROUGH
;
; set MScanTbl Dial Value field based on CurrNumVal
;
CMenuSetNumVal:
	ldd		r16,MenuNumVal
	subi	r16,NUM12HR		;scale to 0 based
	ldiw	Z,NumValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_NumVal
	rjmp	CommonSetVal

CMenuSaveNumVal:
	lds		r17,CurrNumVal
	ldd		r16,MenuNumVal
	cp		r16,r17
	breq	CMenuSNVL600			;no change
	rcall	UpdateNumericDisplay	;needs r16 as input
	rcall	SetCMENULAST
CMenuSNVL600:
	ret

SetCMENULAST:
	ldi		r16,CMENULAST
	std		CMenuState,r16			;set CMenuState to exit menu
	ret

CommonSetVal:
	st		Z,r16
	std		Z+1,r17
	rcall	UpdateCMenuFlashEntry
	ret

CMenuNextCalVal:
	ldd		r16,MenuCalVal
	eor		r16,_1
	std		MenuCalVal,r16
;
; FALL THROUGH
;
CMenuSetCalVal:
	ldd		r16,MenuCalVal			;ShowCalibration
	ldiw	Z,OffOnValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_CalVal
	rjmp	CommonSetVal

CMenuSaveCalVal:
	ldd		r16,MenuCalVal
	ldd		r17,ShowCalibration
	cp		r16,r17
	breq	CMenuSCVL300			;no change
	rcall	UpdateCalibration
	rcall	SetCMENULAST
CMenuSCVL300:
	ret

CMenuNextLedVal:
	ldd		r16,MenuLedVal
	eor		r16,_1
	std		MenuLedVal,r16
;
; FALL THROUGH
;
CMenuSetLedVal:
	ldd		r16,MenuLedVal			;LedDisabled
	ldiw	Z,OnOffValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_LedVal
	rjmp	CommonSetVal

CMenuSaveLedVal:
	ldd		r16,MenuLedVal
	lds		r17,LedDisabled
	cp		r16,r17
	breq	CMenuSLVL300			;no change
	rcall	UpdateLedBlinking
	rcall	SetCMENULAST
CMenuSLVL300:
	ret

CMenuNextBurninVal:
	ldd		r16,MenuBurninVal
	eor		r16,_1
	std		MenuBurninVal,r16
;
; FALL THROUGH
;
CMenuSetBurninVal:
	ldd		r16,MenuBurninVal		;BurnInDisabled
	ldiw	Z,OnOffValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_BurninVal
	rjmp	CommonSetVal

CMenuSaveBurninVal:
	ldd		r16,MenuBurninVal
	lds		r17,BurnInDisabled
	cp		r16,r17
	breq	CMenuSBVL300			;no change
	rcall	UpdateBurnInPrevention
	rcall	SetCMENULAST
CMenuSBVL300:
	ret

#if FUNMODES
CMenuNextPlayVal:
	ldd		r16,MenuPlayVal
	inc		r16
	cpi		r16,FUNFASTR+1	;wrap?
	brlt	NextFVL100
	ldi		r16,FUNNORM		;yes
NextFVL100:
	std		MenuPlayVal,r16
;
; FALL THROUGH
;
CMenuSetPlayVal:
	ldd		r16,MenuPlayVal
	subi	r16,FUNNORM				;scale to 0 based
	ldiw	Z,PlayValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_PlayVal
	rjmp	CommonSetVal


CMenuSavePlayVal:
	ldd		r16,MenuPlayVal
	ldd		r17,CurrPlayMode
	cp		r16,r17
	breq	CMenuSFVL300			;no change
	rcall	UpdPlayMode
	rcall	SetCMENULAST
CMenuSFVL300:
	ret
#endif		; FUNMODES

CMenuNextUsrNameVal:
	lds		r16,MenuUsrNameVal
	inc		r16
	cpi		r16,USRNAMEEDIT+1	;wrap?
	brlt	NextUVL100
	ldi		r16,USRNAMEOFF		;yes
NextUVL100:
	sts		MenuUsrNameVal,r16
;
; FALL THROUGH
;
CMenuSetUsrNameVal:
	lds		r16,MenuUsrNameVal
	subi	r16,USRNAMEOFF			;scale to 0 based
	ldiw	Z,UsrNameValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_UsrNameVal
	rjmp	CommonSetVal

CMenuSaveUsrNameVal:
	lds		r16,MenuUsrNameVal
	cpi		r16,USRNAMEEDIT			;edit mode
	brne	CMenuSUVL200
;
; Edit the UsrName. Make sure CMenuState is advanced to CMENUNAMEDIT
; Start Flashing the first char in the edit buffer
; Do NOT update UsrNameFlg (leave display mode as it was)
;
	std		UsrNameEditIdx,_0		;set current edit char
	rcall	StartMenuTextFlash
	rjmp	CMenuSUVL300
CMenuSUVL200:
	lds		r17,UsrNameFlg
	cp		r16,r17
	breq	CMenuSUVL300			;no change
	rcall	UpdUsrNameMode
	rcall	SetCMENULAST
CMenuSUVL300:
	ret

CMenuIncUserNameChar:
	ldd		r16,UsrNameEditIdx		;current edit char
	ldiw	Z,UsrNameBuf			;look at buffer
	add		ZL,r16					;computer ptr
	adc		ZH,_0
	ld		r16,Z					;get char
#if DEBUG
	tst		r16						;if NULL -> space
	brne	CMenuIUNCL100
	rjmp	PC
CMenuIUNCL100:
#endif
	inc		r16
	sbrc	r16,7					;overflow?
	ldi		r16,SPACE				;reset to SPACE
	st		Z,r16					;save new char
	rcall	CmenuFlashUsrName
	ret
;
; center the UsrName Text line
;
CMenuCenterUsrName:
	ldiw	Z,UsrNameBuf			;must be NULL terminated
	movw	r0,ZL					;save
	ldd		r16,MenuUsrNameCnt		;current length
	add		ZL,r16					;compute ptr
	adc		ZH,_0
	st		Z,_0					;terminate
	movw	ZL,r0					;restore
	rcall	ComputeSRamHPos			;center
	sts		MScanTbl_UsrNameTxt+2,r16
	ret
;
; Build the Alternate buffer for flashing
;
CmenuFlashUsrName:
	ldiw	Z,UsrNameBuf
	ldiw	X,UsrNameFlashBuf
	ldi		r17,MAXUSRNAMELEN	;buffer length
CMenuSUNCL100:
	ld		r16,Z+
	st		X+,r16
	dec		r17
	brne	CMenuSUNCL100
	ldd		r16,UsrNameEditIdx		;current edit char
	ldiw	Z,UsrNameFlashBuf		;compute flash buffer ptr
	add		ZL,r16
	adc		ZH,_0
	ldi		r16,UNDERSCORE			;set char to flash to UNDERSCORE
	st		Z,r16
	ret
;
; MenuUsrNameIdx is 0..MenuUsrNameCnt-1
;
CMenuNextUserNameChar:
	ldd		r16,UsrNameEditIdx		;current edit char
	inc		r16
	std		UsrNameEditIdx,r16		;save. Cnt is now Idx+1
;
; Update the current UsrNameChar Length, if needed
;
	inc		r16						;Idx->Cnt
	ldd		r17,MenuUsrNameCnt		;current length
	cp		r16,r17					;edit length vs current length
	brlt	CMenuNUNCL50			;brle better but no matter
	std		MenuUsrNameCnt,r16		;update
CMenuNUNCL50:
	dec		r16						;Cnt -> Idx
	cpi		r16,MAXUSRNAMELEN		;buffer full?
	brlt	CMenuNUNCL100			;no
;
; Buffer full, exit edit mode
; Remove any trailing spaces
;
CMenuNUNCL90:
	ldiw	Z,UsrNameBuf			;look at buffer
	dec		r16						;to last position entered
	breq	CMenuNUNCL95			;beginning of buffer
	add		ZL,r16					;compute ptr to NULL
	adc		ZH,_0
	ld		r17,Z					;last char
	cpi		r17,SPACE				;was it a SPACE
	brne	CMenuNUNCL95			;no, done
	rjmp	CMenuNUNCL90			;inefficient but easy
CMenuNUNCL95:
	inc		r16
	std		UsrNameEditIdx,r16		;index for next char
	rjmp	CMenuNUNCL300			;exit
;
; buffer not full. See if we had 2 consecutive spaces
;
CMenuNUNCL100:
	ldiw	Z,UsrNameBuf			;look at buffer
	dec		r16						;back 2 positions
	breq	CMenuNUNCL200			;beginning of buffer. Not done
	dec		r16	
	add		ZL,r16					;compute ptr
	adc		ZH,_0
	ld		r17,Z+					;get char
	cpi		r17,SPACE
	brne	CMenuNUNCL200			;not done
	ld		r17,Z+					;next char
	cpi		r17,SPACE
	brne	CMenuNUNCL200
;
; two consecutive spaces. Reset Index, exit
;
	std		UsrNameEditIdx,r16
	rjmp	CMenuNUNCL300
;
; not done. Check next char to see if it is 0
; Make sure we stay in the CMENUNAMEDIT mode
;
CMenuNUNCL200:
	ldd		r16,UsrNameEditIdx		;new index
	ldiw	Z,UsrNameBuf			;buffer ptr
	add		ZL,r16
	adc		ZH,_0
	ld		r16,Z
#if DEBUG
	cpi		r16,0xff				;should never happen
	brne	CMenuNUNCL230
	rjmp	PC
CMenuNUNCL230:
#endif
	tst		r16						;0 (name extension)
	brne	CMenuNUNCL240
	ldi		r16,SPACE				;start at SPACE to exit easier
	st		Z,r16
CMenuNUNCL240:
	rcall	CMenuCenterUsrName
	rcall	CmenuFlashUsrName

#if DEBUG
	ldd		r16,CMenuState
	cpi		r16,CMENUNAMEDIT
	breq	CMenuNUNCL250
	rjmp	PC
CMenuNUNCL250:
#endif
	ldi		r16,CMENUNAMEDIT-1
	std		CMenuState,r16
;	rjmp	CMenuSetUserNameChar	;update buffers info
	ret
;
; Done. Terminate UsrName. Exit edit Mode
;
CMenuNUNCL300:
	ldd		r16,UsrNameEditIdx		;current edit char position
	sts		UsrNameCnt,r16			;reset length
	ldiw	Z,UsrNameBuf			;compute buffer ptr
	add		ZL,r16
	adc		ZH,_0
	st		Z,_0					;terminate UsrName
;
; Assume name was changed. Could optimize here in case there was no change.
;
	sts		UsrNameDflt,_0			;not default any more
	rcall	SaveUsrName				;write to EEProm and update display
	rcall	StopMenuTextFlash
	rcall	SetCMENULAST
	ret

#if GPS
;
; Process GPS option
;
CMenuNextGPSVal:
	lds		r16,MenuGPSVal
	inc		r16
	cpi		r16,GPSOFSPLUS12+1		;wrap?
	brlt	CMenuNGVL100
	ldi		r16,NOGPS				;yes
CMenuNGVL100:
	sts		MenuGpsVal,r16
;
; FALL THROUGH
;
CMenuSetGPSVal:
	lds		r18,MenuGPSVal
	subi	r18,NOGPS				;scale to 0 based
	ldiw	A,TMenuGPSVals*2		;table of GPS vals, 4 bytes each
	lsl		r18						;times 2
	lsl		r18						;times 4
	add		r16,r18
	adc		r17,_0
	ldiw	Z,MScanTbl_GPSVal
	rjmp	CommonSetVal

CMenuSaveGPSVal:
	lds		r16,MenuGPSVal			;desired value
	cpi		r16,NOGPS				;off
	breq	CMenuSGVL200
;
; Desired GPS value is On
;
; Compute desired local time zone offset
;
	subi	r16,(NOGPS+13)
;
; Compare with current value
;
	lds		r17,GPSInEnabled		;get old value
	tst		r17
	breq	CMenuSGVL100			;was off, go change
;
; Desired value is ON, Old value was ON
; Check new local time offset
;
	lds		r17,GPSOffset			;old value
	cp		r16,r17					;same?
	breq	CMenuSGVL400			;yes, no change
;
; set new values
;
CMenuSGVL100:
	sts		GPSOffset,r16
	sts		GPSInEnabled,_1
;
;	Enable GPS input mode
;
	rcall	GPSOnBaudRate
	rjmp	CMenuSGVL250			;save new values
;
; Desired GPS value is Off
;
CMenuSGVL200:
	sts		GPSOffset,_0			;set local time zone offset to 0
	lds		r17,GPSInEnabled		;old value
	tst		r17
	breq	CMenuSGVL400			;was off, no change
	sts		GPSInEnabled,_0			;turn off
	rcall	GPSOffBaudRate
CMenuSGVL250:
	lds		r16,GPSInEnabled
	rcall	UpdateGPS_EEprom
	lds		r16,GPSOffset
	rcall	UpdateGPSOffset_EEprom
;
; Save New Baud Rate in EEprom
;
	lds		r16,BaudVal
	rcall	UpdateBaudrate_EEPprom
	rcall	SetCMENULAST
CMenuSGVL400:
	ret

#else
CMenuNextGPSVal:
CMenuSaveGPSVal:
CMenuSetGPSVal:
	ret

#endif			;GPS

;
; Process DST option
;
CMenuNextDSTVal:
	lds		r16,MenuDSTMode
	inc		r16
	cpi		r16,DSTNONE+1			;wrap?
	brlt	CMenuNDVL100
	ldi		r16,DSTUS				;yes
CMenuNDVL100:
	sts		MenuDSTMode,r16
;
; FALL THROUGH
;
CMenuSetDSTVal:
	lds		r16,MenuDSTMode
	ldiw	Z,DSTValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_DSTVal
	rjmp	CommonSetVal

CMenuSaveDSTVal:
	lds		r16,MenuDSTMode			;desired value
	lds		r17,DSTMode				;current Mode
	cp		r16,r17
	breq	CMenuSDVL400			;same, no change
;
; Update DST mode
;
	rcall	UpdateDSTMode			;r16 is argument
	rcall	SetCMENULAST
CMenuSDVL400:
	ret

CMenuNextChronoVal:
	lds		r16,MenuChronoVal
	eor		r16,_1
	sts		MenuChronoVal,r16
;
; FALL THROUGH
;
CMenuSetChronoVal:
	lds		r16,MenuChronoVal
	ldiw	Z,OffOnValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_ChronoVal
	rjmp	CommonSetVal

CMenuSaveChronoVal:
	lds		r16,MenuChronoVal
	lds		r17,ChronoEnabled
	cp		r16,r17
	breq	CMenuSChrVL300			;no change
	rcall	UpdateChronoDisplay
	rcall	SetCMENULAST
CMenuSChrVL300:
	ret

CMenuNextInitVal:
	lds		r16,MenuEEInitVal
	eor		r16,_1
	sts		MenuEEInitVal,r16
;
; FALL THROUGH
;
CMenuSetInitVal:
	lds		r16,MenuEEInitVal
	ldiw	Z,OffOnValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_InitVal
	rjmp	CommonSetVal

CMenuSaveInitVal:
	lds		r16,MenuEEInitVal
	tst		r16
	breq	CMenuSIVL100			;Init Off, do nothing
	rcall	ClearEEConfiguration
	CALL	InitiateSysReset		;reboot
	rcall	SetCMENULAST
CMenuSIVL100:
	ret

#if MULTIAPP
CMenuNextAppVal:
	lds		r16,MenuAppVal
	inc		r16
	cpi		r16,APPBOOT+1
	brlt	CMenuSAVL100
	ldi		r16,APPCLOCK
CMenuSAVL100:
	sts		MenuAppVal,r16
;
; FALL THROUGH
;
CMenuSetAppVal:
	lds		r16,MenuAppVal
	subi	r16,APPCLOCK			;scale to 0 based
	ldiw	Z,AppValTbl*2
	rcall	GetPointer4Index
	ldiw	Z,MScanTbl_AppVal
	rjmp	CommonSetVal

CMenuSaveAppVal:
	lds		r16,MenuAppVal
	lds		r17,CurrAppMode
	cp		r16,r17
	breq	CMenuSBmVL300			;no change
	sts		CurrAppMode,r16			;set new mode. Could be BOOT
	rcall	SetCMENULAST
CMenuSBmVL300:
	ret
#endif

#if MULTIBAUD
;
; Process BaudRate option
;
CMenuNextBaudVal:
	lds		r16,MenuBaudVal
	inc		r16
	cpi		r16,BAUDVAL14400+1		;wrap?
	brlt	CMenuNBVL100
	ldi		r16,BAUDVALBASE		;yes
CMenuNBVL100:
	sts		MenuBaudVal,r16
;
; FALL THROUGH
;
CMenuSetBaudVal:
	lds		r18,MenuBaudVal
	subi	r18,BAUDVALBASE		;scale to 0 based
	ldiw	A,TMenuBaudVals*2		;table of Baud Rate values, 4 bytes each
	lsl		r18						;times 2
	lsl		r18						;times 4
	lsl		r18						;times 8
	add		r16,r18
	adc		r17,_0
	ldiw	Z,MScanTbl_BaudVal
	st		Z,r16
	std		Z+1,r17
	RCALL	UpdateCMenuFlashEntry
	ret

CMenuSaveBaudVal:
	lds		r16,MenuBaudVal			;desired value
	subi	r16,BAUDVALBASE		;scale to 0 based
	lds		r17,BaudVal				;current value
	cp		r17,r16					;different?
	breq	CMenuSBVL400			;no, out
	sts		BaudVal,r16				;update
	rcall	SetBaudVal				;r16 input
	lds		r16,BaudVal
	rcall	UpdateBaudrate_EEPprom
	rcall	SetCMENULAST			;done
CMenuSBVL400:
	ret
#endif
