; UsrCommand.asm -- Included source for Dutchtronix Oscilloscope Clock
;
;  Copyright  2008 Jan P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Firmware for Dutchtronix AVR Oscilloscope Clock 
;
#if SERIALIO
#if EEPROMHELP
UsrDisplayHelp:
	rcall	ShowClockVersion	;first shared message
	st		-X,_0					;remove CR/LF
	st		-X,_0
	rcall	Special_UARTTX

	ldiw	X,EEPROM_HELP
	rjmp	UsrDHL100
UsrDemoHelp:
	ldiw	X,EEPROM_DEMOHELP
UsrDHL100:
DHL100:
	ldi		r17,1 			;read EEPROM
	rcall	EepromTalk
	tst		r16				;end of text?
	breq	DHL200
	cpi		r16,0xff		;EEPROM not programmed?
	breq	DHL200
	pushw	X				;was auto-incremented
	mov		r24,r16
	rcall	Special_UARTSB	;print one char
	popw	X
	rjmp	DHL100
DHL200:
	rjmp	clctail

#else

UsrDisplayHelp:
	rcall	ShowClockVersion	;first shared message
	st		-X,_0					;remove CR/LF
	st		-X,_0
	rcall	Special_UARTTX

	ldiw	Z,UsrHelpText*2
	rjmp	UsrDHL100
UsrDemoHelp:
	ldiw	Z,UsrDemoHelpText*2
UsrDHL100:
	rcall	Special_UARTPS
	rjmp	clctail
#endif	; EEPROMHELP
;
; Make sure interrupts are ON when calling this function
;
DSTReportAction:
	ldiw	Z,MsgPszDSTApplied*2
	ldiw	X,TXBuffer
	rcall	MsgCopyZToXString
	rcall	DstReportVectors
	rcall	MsgAddNewLine
	st		X+,_0
	rcall	Special_UARTTX		;start Transmitting
	ret
;
; Add current DST Bitvector info to output text buffer (X)
;
; Uses	r16,r24
;
DstReportVectors:
	ldi		r16,'F'			;forward vector
	lds		r24,DSTFYears
	rcall	UsrShowNumericStatus
	ldi		r16,'B'			;backward vector
	lds		r24,DSTBYears
	rcall	UsrShowNumericStatus
	ret
;
; Add a SPACE to output text buffer (X)
;
UsrInsertSpace:
	ldi		r24,SPACE		;separator
	st		X+,r24
	ret
;
; Show Status with Numeric Field.
;	Add to output text buffer (X)
;
; IN	r16 - status char
;		r24 - numeric valule
;
UsrShowNumericStatus:
	st		X+,r16
	RCALL	formathexadecimal
	rcall	UsrInsertSpace
	ret
;
; convert char in r24 to upper case
;
toupper:
	cpi		r24,'a'			;is this a lower case char
	brlt	toupperL100		;no
	cpi		r24,'z'+1
	brsh	toupperL100
	ldi		r16,'a'-'A'		;rescale
	sub		r24,r16
toupperL100:
	ret

;
; Special Versions of the output functions.
; These call the main pump (ClockWorks) while waiting for
; the serial port to be ready.
; Issue: If ClockWorks wants to do output while being called from one
;		 of these. Recursion may occur and order of output may be messed up.
;
; SpecialClockWorks saves all registers.
;
; return char in r24. 'X' means timeout occurred
;

SerialSpecialClockWorks:
	jmp	SpecialClockWorks

SpecialWait4UART:
	rcall	UARTReady			;carry set if ready, uses r0
	brcs	SpecialW4UL800
	rcall	SerialSpecialClockWorks
	rjmp	SpecialWait4UART
SpecialW4UL800:
	ret

Special_UARTRB:
	push	r16
	ldd		r16,AvrSecs
	ldi		r24,INP_TIMEOUT	;timeout seconds 
	add		r16,r24
	cpi		r16,60			;mod 60
	brlt	Special_RBL100
	subi	r16,60
Special_RBL100:
	rcall	UARTHasChar		;Carry set if char
	brcs	Special_RBL200	;Character present
	rcall	SerialSpecialClockWorks
	ldd		r24,AvrSecs
	cp		r16,r24			;if same, timeout
	breq	Special_RBL300	
	rjmp	Special_RBL100
Special_RBL200:
	rcall	UART_ReceiveByte ;get data in r24
	pop		r16
	rjmp	clctail
Special_RBL300:
	rjmp	sectail
;
; char to print in r24
;
Special_UARTSB:
	push	r24
	rcall	SpecialWait4UART
	pop		r24
	rcall	UART_SendByte
	ret
;
; char ptr to print in Z
;
Special_UARTPSZ:
#if SPACESAVINGHACK
	ldi	ZH,high(MsgPszLogo*2)
#endif
Special_UARTPS:
	rcall	SpecialWait4UART
	rcall	UART_PrintfProgStr
	ret
;
; Print TX Buffer
;
Special_UARTTX:
	rcall	SpecialWait4UART
	rcall	UART_PrintTXBuffer
	ret

UsrPrintNewLine:
	ldiw	Z,MsgPszNewLine*2
	rjmp	Special_UARTPSZ
;
; Serial (RS-232) User Interface
;
UsrCommand:
;
; no need to loop here since we know a char is waiting
;
	rcall	UART_ReceiveByte	;get data in r24
	rcall	toupper				;ignore case
	cpi		r24,'X'				;is it an 'X'
	brne	UsrL15				;no, exit
;
; user asks attention by typing 'x' or 'X'
; prompt for command. Answers are case insensitive.
;
UsrL10:
	ldiw	Z,MsgPszCommand*2
	rcall	Special_UARTPS
	rcall	Special_UARTRB		;get command in r24
	rcall	Special_UARTSB		;echo
	rcall	toupper
	rcall	UsrPrintNewLine
	subi	r24,'A'				;rebase
	brlt	UsrL10				;out of range
	cpi		r24,'Z'-'A'+1
	brge	UsrL10				;out of range
	lsl		r24
	ldiw	Z,CmdMapping*2		;Command Mapping table
	add		Zl,r24
	adc		ZH,_0
	lpm		r0,Z+				;get command function ptr
	lpm		r1,Z+
	movw	ZL,r0
	icall						;call the function
	brcc	UsrL15
	ldiw	Z,MsgPszCancel*2
	rcall	Special_UARTPSZ
UsrL15:
	rcall	UsrShowStatus
;
; force complete clock redraw
;
	std		DisplayRedrawFlg,_1
	ret							;end UsrCommand

UsrNoCommand:
clctail:
	clc
	ret

UsrCancel:
sectail:
	sec
	ret

MsgPrintNumericCommand:
	ldiw	Z,2*MsgPszNumericCommand
	rcall	Special_UARTPSZ
	ret

UsrSetTime:
	ldiw	Z,2*MsgPszHourCommand
	rcall	UsrGetNum
	brcs	UsrSTL900
	rcall	AVRSetHours
	cp		r16,r24
	brne	UsrSTL900				;out of range
	rcall	UsrPrintNewLine
	ldiw	Z,2*MsgPszMinCommand
	rcall	UsrGetNum
	brcs	UsrSTL900
	rcall	AVRSetMins
	cp		r16,r24
	brne	UsrSTL900				;out of range
	rcall	UsrPrintNewLine
	ldiw	Z,2*MsgPszSecCommand
	rcall	UsrGetNum
	brcs	UsrSTL900
	rcall	AVRSetSecs
	cp		r16,r24
	brne	UsrSTL900				;out of range
;
; now update the RTC with these new values
;
	rcall	RTCSetTime				;This will also validate the new time
;
; check if the new time is in a DST window. If so, set the proper bitvector flag
; but do not apply a DST correction.
;
	CALL	DSTRetroNotApply
	CALL	UpdateDisplay			;show the new time properly
	rjmp	NLclctail
UsrSTL900:
	CALL	UpdateTimeAndDisplay
UsrSTL950:
	rjmp	sectail

UsrShow:
	lds		r16,Verbose				;reload current time/date from RTC iff verbose & RTC Master mode
	tst		r16
	breq	UsrShow100
	rcall	RTCReadTime				;update Time & Date
	rcall	RTCReadDate
	CALL	UpdateDisplay
UsrShow100:
	rcall	ShowClockVersion		;status will be printed on exit
	rjmp	clctail

UsrSetDate:
	ldiw	Z,2*MsgPszYearCommand
	rcall	UsrGetNum
	brcs	UsrSDL900
	mov		r16,r24
	rcall	AVRSetYear
	cp		r16,r24
	brne	UsrSDL900				;out of range
	rcall	UsrPrintNewLine
	ldiw	Z,2*MsgPszMonthCommand
	rcall	UsrGetNum
	brcs	UsrSDL900
	mov		r16,r24
	RCALL	AVRSetMonth
	cp		r16,r24
	brne	UsrSDL900				;out of range
	rcall	UsrPrintNewLine
	ldiw	Z,2*MsgPszDayCommand
	rcall	UsrGetNum
	brcs	UsrSDL900
	mov		r16,r24
	CALL	AVRSetDay
	cp		r16,r24
	brne	UsrSDL900				;out of range
;
; now update the RTC with new date
;
	rcall	RTCSetDate				;This will also validate the new date
;
; check if the new date is in a DST window
;
	CALL	DSTRetroApply			;will update RTC if doing DST fix.
	CALL	UpdateDisplay			;show the new date properly
	rjmp	NLclctail
UsrSDL900:
	rcall	UsrPrintNewLine
	CALL	UpdateDateAndDisplay
UsrSDL950:
	rjmp	sectail

UsrChangeNumericDisplay:
	rcall	MsgPrintNumericCommand
	ldiw	Z,2*MsgPszNumfieldValues
	rcall	UsrGetNum
	brcs	UsrCNDL900
	cpi		r24,NUMOFF-NUM12HR+1 ;range check
	brsh	UsrCNDL900
	subi	r24,-NUM12HR
	mov		r16,r24
	call	UpdateNumericDisplay
NLclctail:
	rcall	UsrPrintNewLine			;saves r24
	rjmp	clctail
UsrCNDL900:
	rjmp	sectail

UsrChangeDialDisplay:
	rcall	MsgPrintNumericCommand
	ldiw	Z,2*MsgPszDialfieldValues
	rcall	UsrGetNum
	brcs	UsrCDDL900
	cpi		r24,DIALMIN-DIAL12HR+1	;upper limit
	brsh	UsrCDDL900
	subi	r24,-DIAL12HR
	mov		r16,r24
	call	UpdateDialDisplay
UsrCDDL100:
	rjmp	NLclctail
UsrCDDL900:
	rjmp	sectail

UsrChangeDSTOptions:
	rcall	MsgPrintNumericCommand
	ldiw	Z,2*MsgPszDSTfieldValues
	rcall	UsrGetNum
	brcs	UsrCDSTOL900
	cpi		r24,3
	brsh	UsrCDSTOL900
	mov		r16,r24
	rcall	UpdateDSTMode
UsrChangeDSTL500:
	rjmp	NLclctail
UsrCDSTOL900:
	rjmp	sectail

UsrChangeBeamParking:
	rjmp	clctail

UsrChangePlayMode:
#if FUNMODES
	rcall	MsgPrintNumericCommand
	ldiw	Z,2*MsgPszFunfieldValues
	rcall	UsrGetNum
	brcs	UsrCPML900
	cpi		r24,FUNFASTR-FUNNORM+1	; upper limit
	brsh	UsrCPML900
	subi	r24,-FUNNORM
	mov		r16,r24					;must be proper NUMERIC value (FUNNORM .. FUNFASTR)
	rcall	UpdPlayMode
UsrCFML100:
	rjmp	NLclctail
UsrCPML900:
	rjmp	sectail
#else
	rjmp	clctail
#endif		;FUNMODES

UsrShowCopyright:
	ldiw	Z,MsgPszCopyright*2
	rcall	Special_UARTPSZ
	rjmp	clctail

UsrResetEEprom:
	RCALL	ClearEEConfiguration
	CALL	InitiateSysReset
	rjmp	clctail
;
; UsrGetName
;
; Request 20 characters, or 2 consecutive spaces or CR or LF
;
UsrGetName:
	ldiw	Z,2*MsgPszGetName
	rcall	Special_UARTPSZ
	clr		r17				;preserve 
	clr		r18				;blank flag
	ldiw	X,UsrNameBuf	;preserve
UsrGNameL100:
	rcall	Special_UARTRB	;get UPPER CASE char r24
	brcs	UsrGNameL800	;timeout
	rcall	Special_UARTSB	;echo
;
; CR or LF will terminate the string
;
	cpi		r24,CR
	breq	UsrGNameL115
	cpi		r24,LF
	breq	UsrGNameL115
	st		X+,r24			;store
	cpi		r24,' '			;blank?
	brne	UsrGNameL150	;no
;
; first or second blank? Two blanks terminate string.
;
	tst		r18				;first blank flag
	brne	UsrGNameL125	;set, this is second blank
	inc		r18				;no set
	rjmp	UsrGNameL170
UsrGNameL115:
	inc		r17
	rjmp	UsrGNameL350
UsrGNameL125:
	inc		r17				;correct for abnormal loop exit
	rjmp	UsrGNameL200
UsrGNameL150:
	clr		r18				;non blank, clear flag
UsrGNameL170:
	inc		r17				;char cnt
	cpi		r17,20			;limit reached?
	brlt	UsrGNameL100	;no
UsrGNameL200:
;
; remove trailing spaces
;
	ld		r24,-X			;last character entered
	cpi		r24,SPACE
	brne	UsrGNameL300	;no SPACE, done
	dec		r17				;adjust cnt
	brne	UsrGNameL200	;loop
UsrGNameL300:
	adiw	X,1				;terminate
UsrGNameL350:
	st		X,_0
	sts		UsrNameCnt,r17	;save count
;
; BUG in V3.0
; UsrNameDflt was not cleared so SaveUsrName didn't write to EEProm
;
	sts		UsrNameDflt,_0	;not default any more
	CALL	SaveUsrName		;write to EEProm and update display
	rjmp	clctail
UsrGNameL800:				;fail
	CALL	RestoreUsrName
	rjmp	sectail

UsrReboot:
	CALL	InitiateSysReset
	rjmp	clctail

#if MULTIAPP
UsrChangeApp:
	rcall	MsgPrintNumericCommand
	ldiw	Z,2*MsgPszChangeAppValues
	rcall	UsrGetNum
	brcs	UsrCAL900
	cpi		r24,4				;range check
	brsh	UsrCAL900
	subi	r24,-APPCLOCK		;rebase
;
; Did we change application in the menu? We are running Clock.
;
#if DEBUG
	lds		r16,CurrAppMode
	cpi		r16,APPCLOCK
	breq	UMSAVL50
	rjmp	PC
UMSAVL50:
#endif
	cpi		r24,APPTERM
	brne	UMSAVL100
	mov		r16,r24
	subi	r16,APPCLOCK			;rebase to 0
	CALL	UpdateStartApp_EEPprom	;r16 argument
	JMP		RunTerm
UMSAVL100:
	cpi		r24,APPGEN
	brne	UMSAVL200
	JMP		RunFuncGen				;do not save in EEprom
UMSAVL200:
	cpi		r24,APPBOOT
	brne	UMSAVL300
	rcall	InitiateSysReset		;will reload CurrAppMode properly
UMSAVL300:
	rjmp	NLclctail
UsrCAL900:
	rjmp	sectail
#endif
;
;	UsrGetNum
;
;	get a 2 digit number from the Serial Port
;
;	IN	Z -- string ptr to print
;
;	OUT	Carry if cancel or timeout
;		r24 - number
;
;	USED: A, B, r24, r0, r1
;
UsrGetNum:
	rcall	Special_UARTPSZ
UsrGNL100:
	rcall	Special_UARTRB	;get high value in r24
	rcall	Special_UARTSB	;echo
	cpi		r24,'X'			;quit?
	brne	UsrGNL200
UsrGNL190:
	rjmp	sectail
UsrGNL200:
	rcall	UsrChkDigit		;check for valid digit
	brcs	UsrGNL100		;invalid, try again
	mov		r16,r24			;save high digit
UsrGNL250:
	rcall	Special_UARTRB	;get low digit in r24
	rcall	Special_UARTSB	;echo
	cpi		r24,'X'			;quit?
	breq	UsrGNL190
	rcall	UsrChkDigit		;check for valid digit
	brcs	UsrGNL250		;invalid, try again
	ldi		r17,10			;multiply high digit by 10
	mul		r16,r17			;result in r1:r0. r1 always 0
	add		r24,r0			;compose binary value
	rjmp	clctail
;
; check if the digit in r24 is a valid digit (0-9)
; return C if invalid, else return binary value in r24
;
UsrChkDigit:
	cpi		r24,'0'			;in range?
	brlo	UsrCDL300		;no
	cpi		r24,'9'+1		;in range?
	brsh	UsrCDL300		;no
	subi	r24,'0'			;convert to binary
	rjmp	clctail
UsrCDL300:
	rjmp	sectail

ShowClockVersion:
	ldd		r16,TXBufferIndex
	cpi		r16,0xff
	brne	ShowCVL900		;TXBuffer busy
	ldiw	X,TXBuffer
	ldiw	Z,MsgPszLogo*2
	rcall	MsgCopyZToXStringZ
	ldiw	Z,MsgPszSignOn*2
	rcall	MsgCopyZToXStringZ
	ldiw	Z,MsgPszVersion*2
	rcall	MsgCopyZToXStringZ
	rcall	MsgAddNewLine
ShowCVL900:
	ret
;
; Display Current Date and Time based on Internal Variables
; Create the text in the Transmit Buffer, then send
;
; Add optional CR/LF based on r16
;
; Buffer overflow checked at code time (fixed format)
;
DisplayAVRDateTime:
	rjmp	UsrShowStatus


MsgAddNewLine:
	ldiw	Z,MsgPszNewLine*2
	rcall	MsgCopyZToXStringZ
	ret


MsgCopyZToXStringZ:
#if SPACESAVINGHACK
	ldi	ZH,high(MsgPszSignOn*2)
#endif
MsgCopyZToXString:
MsgCZTXL100:
	lpm		r0,Z+				;copy string
	st		X+,r0
	tst		r0
	brne	MsgCZTXL100
	sbiw	X,1					;remove NULL
	ret

UsrShowAll:
	rcall	ShowClockVersion
	rjmp	UsrShowSL100
UsrShowStatus:
	ldiw	X,TXBuffer
UsrShowSL100:
	ldd		r16,TXBufferIndex
	cpi		r16,0xff
	rjne	UsrSSL900			;TXBuffer busy
	CALL	FormatAVRDate
	rcall	UsrInsertSpace
	CALL	FormatAVRTime
;
; Space Saving Suggestion:
;	Allocate a static SRam buffer with status fields. These fields
;	need to be dynamically updated as they change.
;	This routine does nothing but copy the buffer the TX buffer
;
	rcall	UsrInsertSpace
;
; display 'L' if Led is on
;
	lds		r16,LedDisabled
	tst		r16
	brne	UsrShowL110		;Disabled, skip Display
	ldi		r24,'L'
	st		X+,r24
UsrShowL110:
;
; display 'B' if BurninPrevention is on
;
	lds		r16,BurnInDisabled
	tst		r16
	brne	UsrShowL120		;Disabled, skip Display
	ldi		r24,'B'
	st		X+,r24
UsrShowL120:
;
; display 'V' if verbose mode
;
	lds		r16,Verbose
	tst		r16
	breq	UsrShow_L200
	ldi		r24,'V'
	st		X+,r24
UsrShow_L200:
;
; display 'U' or 'E' if Daylight Saving Time is on
;
	lds		r16,DSTMode
	cpi		r16,DSTNONE
	breq	UsrShow_L300
	ldi		r24,'U'			;preset
	tst		r16				;DSTUS == 0
	breq	UsrShowL270
	ldi		r24,'E'
UsrShowL270:
	st		X+,r24
UsrShow_L300:
;
; display 'K' if Chrono Display is on
;
	lds		r16,ChronoEnabled
	tst		r16
	breq	UsrShow_L400
	ldi		r24,'K'
	st		X+,r24
UsrShow_L400:
#if DEBUG		;only output command line available if GPS input enabled
;
; display 'G' if GPS Input is on
;
	lds		r16,GPSInEnabled
	tst		r16
	breq	UsrShow_L500
	ldi		r24,'G'
	st		X+,r24
	lds		r24,GPSOffset	;add local time zone offset
	RCALL	formathexadecimal
	rcall	UsrInsertSpace
UsrShow_L500:
#endif
;
; display 'F' if any Fun Mode is enabled
;
	ldd		r16,CurrPlayMode
	cpi		r16,FUNNORM
	breq	UsrShow_L600
	ldi		r24,'F'
	st		X+,r24
UsrShow_L600:

;
; display 'C' if Calibration is on
;
	ldd		r16,ShowCalibration
	tst		r16
	breq	UsrShow_L800
	ldi		r24,'C'
	st		X+,r24
UsrShow_L800:
	rcall	UsrInsertSpace
;
; display DST bitvectors
;
	rcall	DstReportVectors
;
; display 'M' Trigger Mode
;
	ldi		r16,'M'
	lds		r24,PPSTrigger
	rcall	UsrShowNumericStatus
	rcall	MsgAddNewLine
	st		X+,_0				;Termination Char
	rcall	Special_UARTTX		;start Transmitting
UsrSSL900:
	ret
#endif		; SERIALIO
