; dac.asm - included source for Dutchtronix Oscilloscope Clock
;
;  Copyright  2008 Johannes P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Firmware for Dutchtronix AVR Oscilloscope Clock
;
; TLC7528 Dual 8-bit DAC
;
; Protocol: DACSELECT, out 8 Data bits, pulse WR low
; 
DACinit:
#if Mega32
	ser		r16
	out		DACDATADDR,r16
#elif (Mega168 | Mega328p)
;
; Upper 4 bits can be safely ignored here.
; PB4, PB5 are only used for ISP programming. Set as input ports
; PB6, PB7 are unconnected
;  
	sbi		DACHIDDR,0		;high nibble
	sbi		DACHIDDR,1
	sbi		DACHIDDR,2
	sbi		DACHIDDR,3
;
; Upper 4 bits can be safely ignored here.
; PC4, PC5 are used for I2C access to the RTC but are set as input ports otherwise
; PC6, PC7 are unconnected/non-existant
;  
	sbi		DACLODDR,0		;low nibble
	sbi		DACLODDR,1
	sbi		DACLODDR,2
	sbi		DACLODDR,3
#endif
	sbi		DACCTLDDR,3		;DAC Select
	sbi		DACCTLDDR,4		;WR 
	sbi		DACCTLDDR,5		;LDAC control
	sbi		DACLDAC			;
	sbi		DACZXDDR		;Z-Axis control
	sbi		DACWR			;hold
	BEAMOFF
	ret
;
; DACXYPos		X position in r24, Y position in r22
;
; USED:	r21
;
DACXYPos:
;
; X first (DACA)
;
	mov		r21,r24			;preserve r24
#if INVERT
	neg		r21				;map X origin
#endif
	add		r21,GXOffset	;grid offset
	add		r21,XOffset		;Scan Table offset
	cbi		DACSELECT		;select DACA (active low)
#if Mega32
	out		DACDATA,r21		;X
#elif (Mega168 | Mega328p)
	out		DACLODATA,r21
	swap	r21
	out		DACHIDATA,r21	;no need to preserve High nibble of DACHIDATA
#endif
	cbi		DACWR			;latch DACA data
	sbi		DACWR			;hold
;
; Y second (DACB)
;
	mov		r21,r22			;preserve r22 by using r21
#if INVERT
	neg		r21				;map Y origin
#endif
	add		r21,GYOffset	;grid offset
	add		r21,YOffset		;Scan Table offset
	sbi		DACSELECT		;select DACB (active high)
#if Mega32
	out		DACDATA,r21		;Y
#elif (Mega168 | Mega328p)
	out		DACLODATA,r21
	swap	r21
	out		DACHIDATA,r21	;no need to preserve High nibble of DACHIDATA
#endif
	cbi		DACWR			;latch DACB data
	sbi		DACWR			;hold
	cbi		DACLDAC			;update DAC channels
	sbi		DACLDAC
	ret
;
; DACYXPos		Y position in r22, X position in r24
;
; USED:	r21
;
DACYXPos:
;
; Y first (DACA)
;
	mov		r21,r22			;save r22
#if INVERT
	neg		r21				;map Y origin
#endif
	add		r21,GXOffset	;grid offset
	add		r21,XOffset		;Scan Table offset
	cbi		DACSELECT		;select DACA (active low)
#if Mega32
	out		DACDATA,r21		;Y
#elif (Mega168 | Mega328p)
	out		DACLODATA,r21
	swap	r21
	out		DACHIDATA,r21	;no need to preserve High nibble of DACHIDATA
#endif
	cbi		DACWR			;latch DACA data
	sbi		DACWR			;hold
;
; X second (DACB)
;
	mov		r21,r24			;preserve r24 by using r21
#if INVERT
	neg		r21				;map X origin
#endif
	add		r21,GYOffset	;grid offset
	add		r21,YOffset		;Scan Table offset
	sbi		DACSELECT		;select DACB (active high)
#if Mega32
	out		DACDATA,r21		;X
#elif (Mega168 | Mega328p)
	out		DACLODATA,r21
	swap	r21
	out		DACHIDATA,r21	;no need to preserve High nibble of DACHIDATA
#endif
	cbi		DACWR			;latch DACB data
	sbi		DACWR			;hold
	cbi		DACLDAC			;update DAC channels
	sbi		DACLDAC
	ret
;
; Display a Char Pixel (n points in X direction, m points in Y direction)
; where n and m are both either 1 or 2
;
; IN:	r24 (X), r22 (Y)
;
; Uses: r3, r4
;
CharPixel:
	mov		r3,r24
	mov		r4,r22
;
; X first (DACA)
;
	lsl		r24			;times 2
	add		r24,GXOffset ;grid offset
	add		r24,XOffset ;Scan Table offset
;
; Y second (DACB)
;
	lsl		r22			;times 2
	add		r22,GYOffset ;grid offset
	add		r22,YOffset ;Scan Table offset
;
; Here is decided how to draw one char pixel. The
; current design draws 2 pixels. It doesn't matter
; which axis is used (x or y).
;
; Drawing 3 or 4 screen pixels per char pixel will
; give a marginally better picture at a significantly
; reduced refresh rate.
;
#if DRGcode
	rcall	DoXY
#endif


#if (DOTSPERCHARPIXEL==4)
	rcall	DoXY
#if DRGcode
	BEAMON
#endif
	inc		r24
	rcall	DoXY
	dec		r24
	inc		r22
	rcall	DoXY
	inc		r24
	rcall	DoXY
#elif (DOTSPERCHARPIXEL==3)
	rcall	DoXY
	inc		r22
	rcall	DoXY
	inc		r24
	rcall	DoXY
#elif (DOTSPERCHARPIXEL==2)
#if DRGcode
	rcall	DoXY
	BEAMON
#endif
	rcall	DoXY
	inc		r22
#if DRGcode
	rcall	DoXY
#endif
	rcall	DoXY
#else
unsupported
#endif
#if DRGcode
	BEAMOFF
#endif
	mov		r22,r4
	mov		r24,r3
	ret
;
; IN: r24, r22
;
DoXY:	
	cbi		DACSELECT		;select DACA (active low)
#if Mega32
	out		DACDATA,r24
#elif (Mega168 | Mega328p)
	out		DACLODATA,r24	;no need to preserve High nibble of DACLODATA
	swap	r24
	out		DACHIDATA,r24	;no need to preserve High nibble of DACHIDATA
	swap	r24				;restore
#endif
	cbi		DACWR			;latch DACA data
	sbi		DACWR			;hold
	sbi		DACSELECT		;select DACB (active high)
#if Mega32
	out		DACDATA,r22
#elif (Mega168 | Mega328p)
	out		DACLODATA,r22	;no need to preserve High nibble of DACLODATA
	swap	r22
	out		DACHIDATA,r22	;no need to preserve High nibble of DACHIDATA
	swap	r22				;restore
#endif
	cbi		DACWR			;latch DACB data
	sbi		DACWR			;hold
	cbi		DACLDAC			;update DAC channels
	sbi		DACLDAC
	ret

