; demomode.asm - included source for Dutchtronix Oscilloscope Clock
;
;  Copyright  2008 Johannes P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Firmware for Dutchtronix AVR Oscilloscope Clock
;

.macro	SetComment
	ldi		ZL,low(@0*2)
	ldi		ZH,high(@0*2)				;ZH probably stays the same during the demo
	rcall	DemoSetUsrName
.endm


.macro	Wait4
	rcall	Special_Wait4
.endm

.macro	Wait8
	Wait4
	Wait4
.endm

SaveAndSetState:
;
; Assume for now that the "Push to Start" message is not used in Demo Mode
;
	ldiw	Z,ScanTbl_Str1
	ldiw	X,DemoText*2	;Demo Message
	st		Z+,XL
	st		Z+,XH
	ldi		r16,DEMOHPOS
	st		Z+,r16
	ldi		r16,DEMOVPOS			
	st		Z+,r16

	RCALL	DisablePlayModes
	rcall	DisableCalibration
	rcall	DisableChrono
	ldiw	Z,SpecialMainClockWorks	;restore default func ptr
	stdw	fpSpecialClockWorks,Z
	rcall	DemoSetPMTime
	SetComment MsgPszLogo
	rcall	SetDefaultDemoState
	ret

SetDefaultDemoState:
	rcall	Enable12HRDial
	rcall	Enable12HRNumbers
	Wait4
	ret

DisableNumbers:
	ldi		r16,NUMOFF
	rjmp	DM950
Enable12HRNumbers:
	ldi		r16,NUM12HR
	rjmp	DM950
Enable24HRNumbers:
	ldi		r16,NUM24HR
	rjmp	DM950
EnableHexNumbers:
	ldi		r16,NUMHEX
	rjmp	DM950
EnableDateNumbers:
	ldi		r16,NUMDATE
DM950:
	CALL	UpdateNumericDisplay
	rjmp	DM999

Enable12HRDial:
	ldi		r16,DIAL12HR
	rjmp	DM975
Enable24HRDial:
	ldi		r16,DIAL24HR
	rjmp	DM975
EnableMinDial:
	ldi		r16,DIALMIN
	rjmp	DM975
EnableBinDial:
	ldi		r16,DIALBIN
	rjmp	DM975
EnableRomanDial:
	ldi		r16,DIALROMAN
DM975:
	CALL	UpdateDialDisplay
	rjmp	DM999

DisableLedBlinking:
	mov		r16,_1				;disabled
	rjmp	EnableLBL100
EnableLedBlinking:
	clr		r16					;not disabled
EnableLBL100:
	RCALL	UpdateLedBlinking	;won't update EEProm
	rjmp	DM999
DisableCalibration:
	clr		r16
	rjmp	EnableCL100
EnableCalibration:
	mov		r16,_1
EnableCL100:
	RCALL	UpdateCalibration	;won't update EEProm
	rjmp	DM999
DisableChrono:
	clr		r16
	rjmp	EnableChL100
EnableChrono:
	mov		r16,_1
EnableChL100:
	RCALL	UpdateChronoDisplay	;won't update EEProm
	rjmp	DM999
DM999:
;	rjmp	DM1000
;DM1000:
	ret
;
; This code depends on the fact that the Dial normally cannot flash. May change.

EnableFlashingDial:
	lddw	X,DialFlashPtr			;FlashTbl entry
#if DEBUG
	mov		r16,XL					;skip if already active
	or		r16,XH
	breq	DialFlashingOnL100
	rjmp	PC
DialFlashingOnL100:
#endif
	ldiw	X,ScanTbl_Dial
	ldiw	Z,DialData*2			;Dial
	ldi		r16,FLASHPERSEC*20		;20 seconds
	rcall	AddFlashItemNoAlt
	stdw	DialFlashPtr,X			;needed to turn PPS missing signal off
	ret

DisableFlashingDial:
	lddw	X,DialFlashPtr			;FlashTbl entry
	rcall	RemoveFlashItem
	clrdw	DialFlashPtr			;clear current ptr
	ret
;
; Previous state of RotationIndex not saved here. OK since demo will restart
;
#define EnableBurnInPerSecond		std		RotationIndex,_0

GPS8SecsWarning:
	ldi		r16,FLASHPERSEC*8		;# of seconds
	CALL	GPSStartFlashing	
	ret

Battery8SecsWarning:
	ldi		r16,FLASHPERSEC*8		;# of seconds
	CALL	BatteryStartFlashing	
	ret

PPS8SecsWarning:
	ldi		r16,FLASHPERSEC*8		;# of seconds
	CALL	PPSStartFlashing	
	ret

DemoNumericField:
	rcall	Enable12HRNumbers
	Wait4
	rcall	EnableDateNumbers
	Wait4
	rcall	Enable24HRNumbers
	Wait4
	rcall	EnableChrono
	Wait4
	rcall	disableChrono
	rcall	EnableHexNumbers
	Wait4
	rcall	Enable12HRNumbers
	Wait4
	rcall	DisableNumbers
	Wait4
	ret

DemoSetPMTime:
	stdi	AvrHrs,15			;3:44PM, to make sure the 24hr clock is displayed
	stdi	AvrMins,42			;also to highlight Hex Display
	CALL	UpdateDisplay
	ret
;
; DemoNumericField with Binary Clock
; Playmode with Binary Clock
; many blank lines on console
;
UsrDemo:
	ldi		r16,(1<<fDemoGoing)
	sts		DemoEnabled,r16
	rcall	SaveAndSetState
#if	SERIALIO
	rcall	UsrPrintNewLine
	rcall	UsrDemoHelp
#endif
	rcall	DemoNumericField

	SetComment	TYourName

	rcall	Enable12HRDial
	Wait4
	rcall	EnableMinDial
	Wait4
	rcall	Enable24HRDial			;depends on the time
	Wait4
	rcall	EnableRomanDial
	Wait4

	SetComment	TBinClock

	rcall	EnableBinDial
	Wait4
	rcall	DemoNumericField

	SetComment	TBiHexClock
	rcall	EnableHexNumbers
	Wait4

	rcall	Enable12HRDial
	rcall	Enable12HRNumbers
	SetComment	T30Combinations
	Wait4
;
; mix and match
;  comment in Binary Clock and hex display
;
	SetComment	TYourName
	Wait4
	rcall	EnableChrono
	rcall	Enable24HRDial
	Wait4
	rcall	DisableChrono
	rcall	EnableDateNumbers
	Wait4
	rcall	EnableRomanDial
	Wait4
	rcall	EnableBinDial
	rcall	Enable12HRNumbers
	Wait4
	rcall	EnableMinDial
	rcall	EnableHexNumbers
	Wait4
	rcall	Enable12HRNumbers
	rcall	Enable12HRDial

	SetComment	TLedClock
	rcall	DisableLedBlinking
	Wait4
	rcall	EnableLedBlinking
	Wait4
	SetComment	TCalClock
	Wait4
	rcall	EnableCalibration
	Wait4
	rcall	DisableCalibration
	Wait4
;
; Fun stuff
;
	SetComment	TFunClock
	ldi		r16,FUNFASTF
	CALL	UpdPlayMode
	Wait8
	CALL	DisablePlayModes
	Wait4
	ldi		r16,FUNREV
	CALL	UpdPlayMode
	Wait8
	ldi		r16,FUNFASTR
	CALL	UpdPlayMode
	Wait8
	CALL	DisablePlayModes

	SetComment	TWarnClock
	rcall	Battery8SecsWarning		;temporary flashing
	Wait8
	stsi	DemoEnabled,((1<<fDemoPPS)|(1<<fDemoGoing))
	rcall	PPS8SecsWarning			;temporary flashing
	Wait8
	stsi	DemoEnabled,(1<<fDemoGoing)	;turn off fDemoPPS
	rcall	GPS8SecsWarning			;temporary flashing
	Wait8

	rcall	EnableFlashingDial
	Wait8
	rcall	DisableFlashingDial

	SetComment	TBurnClock
	stsi	DemoEnabled,((1<<fDemoBurnIn)|(1<<fDemoGoing))
	EnableBurnInPerSecond
	Wait8
	stsi	DemoEnabled,(1<<fDemoGoing)		;turn off fDemoBurnIn

	SetComment	TScreenMenu
	CALL	CMenuSetTable
	ldiw	Z,SpecialMenuClockWorks	;set Menu func ptr
	stdw	fpSpecialClockWorks,Z
	Wait8
	ldiw	Z,SpecialMainClockWorks	;restore default func ptr
	stdw	fpSpecialClockWorks,Z

	SetComment	TNotShown
	Wait4
	SetComment	TGPSClock
	Wait4
	SetComment	TDaylighClock
	Wait4
;
; Demo Change Time and Date
; Demo Automatic Daylight Saving Time adjustment
; Demo GPS NMEA parsing
; Demo Name: first Dutchtronix on,off,on. Then set "Your name here"
; Set to to PM for 24hr dial
;
;	rcall	DisableNumbers
;	rcall	Enable12HRDial
;	rcall	StartFlashingHRHand
;	Wait4
;	rcall	StartFlashingMinHand
;	Wait4
;	rcall	StartFlashingSecHand
;	Wait4
;	rcall	EnableBinDial
;
	rcall	SetDefaultDemoState
	SetComment	TThanksClock
	Wait8
;	rcall	RestoreState				;not needed since a reboot follows
	CALL	InitiateSysReset			;reboot
	ret

TYourName:	.db	"Your Name here",0,0
TBinClock:	.db	"Binary Clock",0,0
TFunClock:	.db	"Play Mode",0
TLedClock:	.db	"LED Control",0
TWarnClock:	.db	"Warnings",0,0
TScreenMenu: .db "On Screen Menu",0,0	
TThanksClock:.db	"Thank You",0

TBiHexClock:.db	"Bi-Hex Clock",0,0
TCalClock:	.db	"Calibration Screen",0,0
TBurnClock:	.db	"BurnIn Prevention",0
TNotShown:	.db	"Also Supported",0,0
TGPSClock:	.db	"GPS NMEA Parsing",0,0
TDaylighClock: .db "Daylight Saving Time",0,0
T30Combinations: .db "30 Combinations",0

